@@ -115,15 +115,13 @@ position_dodgenudge <-
                                      split = sign,
                                      split.y = function(x) {1},
                                      split.x = sign,
-                                     center = sign,
-                                     function(x) {1}),
+                                     center = sign),
                      .fun_y = switch(direction,
                                      none = function(x) {1},
                                      split = sign,
                                      split.x = function(x) {1},
                                      split.y = sign,
-                                     center = sign,
-                                     function(x) {1}),
+                                     center = sign),
                      kept.origin = kept.origin,
                      width = width,
                      preserve = rlang::arg_match(preserve)
