@@ -51,6 +51,16 @@
 #'   second value for the right tail (or if using \code{orientation = "y"} the
 #'   lower and upper tails, respectively).
 #'
+#'   Computation of density and of the default bandwidth require at least
+#'   two observations with different values. If data do not fulfill this
+#'   condition, they are kept only if \code{keep.fraction = 1}. This is correct
+#'   behavior for a single observation, but can be surprising in the case of
+#'   multiple observations.
+#'
+#'   Parameters \code{keep.these} and \code{exclude.these} make it possible to
+#'   force inclusion or exclusion of labels after the density is computed.
+#'   In case of conflict, \code{exclude.these} overrides \code{keep.these}.
+#'
 #' @note Which points are kept and which not depends on how dense and flexible
 #'   is the density curve estimate. This depends on the values passed as
 #'   arguments to parameters \code{n}, \code{bw} and \code{kernel}. It is
@@ -73,10 +83,15 @@
 #' @param keep.sparse logical If \code{TRUE}, the default, observations from the
 #'   more sparse regions are retained, if \code{FALSE} those from the densest
 #'   regions.
-#' @param keep.these character vector, integer vector, logical vector or
-#'   function that takes the variable mapped to the \code{label} aesthetic as
-#'   first argument and returns a character vector or a logical vector. These
-#'   rows from \code{data} are selected irrespective of the local density.
+#' @param keep.these,exclude.these character vector, integer vector, logical
+#'   vector or function that takes one or more variables in data selected by
+#'   \code{these.target}. Negative integers behave as in R's extraction methods.
+#'   The rows from \code{data} indicated by \code{keep.these} and
+#'   \code{exclude.these} are kept or excluded irrespective of the local
+#'   density.
+#' @param these.target character, numeric or logical selecting one or more
+#'   column(s) of \code{data}. If \code{TRUE} the whole \code{data} object is
+#'   passed.
 #' @param pool.along character, one of \code{"none"} or \code{"x"},
 #'   indicating if selection should be done pooling the observations along the
 #'   \emph{x} aesthetic, or separately on either side of \code{xintercept}.
@@ -240,20 +255,25 @@ stat_dens1d_labels <-
            keep.number = Inf,
            keep.sparse = TRUE,
            keep.these = FALSE,
-           pool.along = "x",
+           exclude.these = FALSE,
+           these.target = "label",
+           pool.along = c("x", "none"),
            xintercept = 0,
            invert.selection = FALSE,
            bw = "SJ",
            kernel = "gaussian",
            adjust = 1,
            n = 512,
-           orientation = "x",
+           orientation = c("x", "y"),
            label.fill = "",
            return.density = FALSE,
            na.rm = TRUE,
            show.legend = FALSE,
            inherit.aes = TRUE) {
 
+    pool.along <- rlang::arg_match(pool.along)
+    orientation <- rlang::arg_match(orientation)
+
     if (length(label.fill) > 1L) {
       stop("Length for 'label.fill' is not 0 or 1: ", label.fill)
     }
@@ -275,6 +295,8 @@ stat_dens1d_labels <-
                     keep.number = keep.number,
                     keep.sparse = keep.sparse,
                     keep.these = keep.these,
+                    exclude.these = exclude.these,
+                    these.target = these.target,
                     pool.along = pool.along,
                     xintercept = xintercept,
                     invert.selection = invert.selection,
@@ -289,121 +311,6 @@ stat_dens1d_labels <-
     )
   }
 
-dens1d_labs_compute_fun <-
-  function(data,
-           scales,
-           keep.fraction,
-           keep.number,
-           keep.sparse,
-           keep.these,
-           pool.along,
-           xintercept,
-           invert.selection,
-           bw,
-           kernel,
-           adjust,
-           n,
-           orientation,
-           label.fill,
-           return.density) {
-
-    force(data)
-    if (!exists("label", data) && !is.null(label.fill)) {
-      data[["label"]] <- rownames(data)
-    }
-
-    keep.these <- keep_these2logical(keep.these = keep.these, data = data)
-
-    # discard redundant splits and make list of logical vectors
-    if (pool.along != "x" &&
-        xintercept < max(data[[orientation]]) &&
-        xintercept > min(data[[orientation]])) {
-
-      selectors <-list(low.tail = data[[orientation]] <= xintercept,
-                       high.tail = data[[orientation]] > xintercept)
-      if (length(keep.fraction) != 2L) {
-        keep.fraction <- rep_len(keep.fraction, length.out = 2)
-      }
-      if (length(keep.number) != 2L) {
-        if (length(keep.number) == 1L) {
-          keep.number <- keep.number %/% 2
-        }
-        keep.number <- rep_len(keep.number, length.out = 2)
-      }
-      num.rows <- sapply(selectors, sum) # selectors are logical
-    } else {
-      keep.fraction <- keep.fraction[[1]] # can be a vector or a list
-      keep.number <- keep.number[[1]]
-      num.rows <- nrow(data)
-      selectors <- list(all = rep.int(TRUE, times = num.rows))
-    }
-
-    # vectorized
-    too.large.frac <- num.rows * keep.fraction > keep.number
-    keep.fraction[too.large.frac] <-
-      keep.number[too.large.frac] / num.rows[too.large.frac]
-
-    # density on a grid
-    dens <-
-      stats::density(data[[orientation]],
-                     bw = bw, kernel = kernel, adjust = adjust, n = n,
-                     from = scales[[orientation]]$dimension()[1],
-                     to = scales[[orientation]]$dimension()[2])
-
-    # estimate density at each observations coordinates
-    fdens <- stats::splinefun(dens$x, dens$y) # y contains estimate of density
-    dens <- fdens(data[[orientation]])
-
-    # we construct one logical vector by adding observations/label to be kept
-    # we may have a list of 1 or 2 logical vectors
-    keep <- keep.these
-    for (i in seq_along(selectors)) {
-      if (keep.fraction[i] == 1) {
-        keep[ selectors[[i]] ] <- TRUE
-      } else if (keep.fraction[i] != 0) {
-        if (keep.sparse) {
-          keep[ selectors[[i]] ] <-
-            keep[ selectors[[i]] ] |
-            dens[ selectors[[i]] ] < stats::quantile(dens[ selectors[[i]] ],
-                                                     keep.fraction[i], names = FALSE)
-        } else {
-          keep[ selectors[[i]] ] <- keep[ selectors[[i]] ] |
-            dens[ selectors[[i]] ] >= stats::quantile(dens[ selectors[[i]] ],
-                                                      1 - keep.fraction[i], names = FALSE)
-        }
-      }
-    }
-
-    if (invert.selection){
-      keep <- !keep
-    }
-
-    if (return.density) {
-      data[["keep.obs"]] <- keep
-      data[["density"]] <- dens
-    }
-
-    if (is.null(label.fill)) {
-      data <- data[keep, ]
-    } else if (is.function(label.fill)) {
-      data[["label"]][!keep] <- label.fill(data[["label"]][!keep])
-    } else if (is.na(label.fill)) {
-      # NA_logical_, the default NA, cannot always be assigned to character
-      label.fill <- NA_character_
-      data[["label"]][!keep] <- label.fill
-    } else if (is.character(label.fill)) {
-      data[["label"]][!keep] <- label.fill
-    } else if (is.logical(label.fill)) {
-      if (label.fill) {
-        data[["label"]][!keep] <- ""
-      } # if FALSE data is not modified
-    } else {
-      stop("'label.fill' is : ", mode(label.fill),
-           " instead of 'character' or 'function'.")
-    }
-    data
-  }
-
 #' @rdname ggpp-ggproto
 #' @format NULL
 #' @usage NULL
@@ -413,6 +320,138 @@ StatDens1dLabels <-
     "StatDens1dLabels",
     ggplot2::Stat,
     compute_panel =
-      dens1d_labs_compute_fun,
+      function(data,
+               scales,
+               keep.fraction,
+               keep.number,
+               keep.sparse,
+               keep.these,
+               exclude.these,
+               these.target = "label",
+               pool.along,
+               xintercept,
+               invert.selection,
+               bw,
+               kernel,
+               adjust,
+               n,
+               orientation,
+               label.fill,
+               return.density) {
+
+        force(data)
+        if (!exists("label", data) && !is.null(label.fill)) {
+          data[["label"]] <- rownames(data)
+        }
+
+        keep.these <- these2logical(these = keep.these,
+                                    data = data,
+                                    these.target = these.target)
+
+        exclude.these <- these2logical(these = exclude.these,
+                                       data = data,
+                                       these.target = these.target)
+
+        # discard redundant splits and make list of logical vectors
+        if (pool.along != "x" &&
+            xintercept < max(data[[orientation]]) &&
+            xintercept > min(data[[orientation]])) {
+
+          selectors <-list(low.tail = data[[orientation]] <= xintercept,
+                           high.tail = data[[orientation]] > xintercept)
+          if (length(keep.fraction) != 2L) {
+            keep.fraction <- rep_len(keep.fraction, length.out = 2)
+          }
+          if (length(keep.number) != 2L) {
+            if (length(keep.number) == 1L) {
+              keep.number <- keep.number %/% 2
+            }
+            keep.number <- rep_len(keep.number, length.out = 2)
+          }
+          num.rows <- sapply(selectors, sum) # selectors are logical
+        } else {
+          keep.fraction <- keep.fraction[[1]] # can be a vector or a list
+          keep.number <- keep.number[[1]]
+          num.rows <- nrow(data)
+          selectors <- list(all = rep.int(TRUE, times = num.rows))
+        }
+
+        # vectorized
+        too.large.frac <- num.rows * keep.fraction > keep.number
+        keep.fraction[too.large.frac] <-
+          keep.number[too.large.frac] / num.rows[too.large.frac]
+
+        # density on a grid
+        # data with fewer than 2 rows is as a special case as density() fails
+        if (length(unique(data[[orientation]])) >= 2L) {
+          dens <-
+            stats::density(data[[orientation]],
+                           bw = bw, kernel = kernel, adjust = adjust, n = n,
+                           from = scales[[orientation]]$dimension()[1],
+                           to = scales[[orientation]]$dimension()[2])
+
+          # estimate density at each observations coordinates
+          fdens <- stats::splinefun(dens$x, dens$y) # y contains estimate of density
+          dens <- fdens(data[[orientation]])
+        } else {
+          if (nrow(data) > 1L) {
+            message("Density not computed, too few distinct values in '", orientation, "'")
+          }
+          dens <- rep_len(1, nrow(data))
+        }
+        # we construct one logical vector by adding observations/label to be kept
+        # we may have a list of 1 or 2 logical vectors
+        keep <- logical(nrow(data))
+        for (i in seq_along(selectors)) {
+          if (keep.fraction[i] == 1) {
+            keep[ selectors[[i]] ] <- TRUE
+          } else if (keep.fraction[i] != 0 && length(selectors[[i]]) >= 2L) {
+            if (keep.sparse) {
+              keep[ selectors[[i]] ] <-
+                dens[ selectors[[i]] ] < stats::quantile(dens[ selectors[[i]] ],
+                                                         keep.fraction[i],
+                                                         names = FALSE,
+                                                         type = 8)
+            } else {
+              keep[ selectors[[i]] ] <-
+                dens[ selectors[[i]] ] >= stats::quantile(dens[ selectors[[i]] ],
+                                                          1 - keep.fraction[i],
+                                                          names = FALSE,
+                                                          type = 8)
+            }
+          }
+        }
+        keep <- (keep | keep.these) & !exclude.these
+
+        if (invert.selection){
+          keep <- !keep
+        }
+
+        if (return.density) {
+          data[["keep.obs"]] <- keep
+          data[["density"]] <- dens
+        }
+
+        if (is.null(label.fill)) {
+          data <- data[keep, ]
+        } else if (is.function(label.fill)) {
+          data[["label"]][!keep] <- label.fill(data[["label"]][!keep])
+        } else if (is.na(label.fill)) {
+          # NA_logical_, the default NA, cannot always be assigned to character
+          label.fill <- NA_character_
+          data[["label"]][!keep] <- label.fill
+        } else if (is.character(label.fill)) {
+          data[["label"]][!keep] <- label.fill
+        } else if (is.logical(label.fill)) {
+          if (label.fill) {
+            data[["label"]][!keep] <- ""
+          } # if FALSE data is not modified
+        } else {
+          stop("'label.fill' is : ", mode(label.fill),
+               " instead of 'character' or 'function'.")
+        }
+        data
+      },
+
     required_aes = "x|y"
   )
